package com.steerinfo.dil.controller;

import com.steerinfo.dil.service.impl.BmsshipStatementServiceImpl;
import com.steerinfo.dil.service.impl.BmstruckFormulaServiceImpl;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.PageListAdd;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/6 17:34
 * @Version:V2.0
 */
@RestController
@RequestMapping("/${api.version}/bmsshipstatement")
public class BmsshipStatementController extends BaseRESTfulController {

    @Autowired
    BmsshipStatementServiceImpl bmsshipStatementService;
    @Autowired
    BmstruckFormulaServiceImpl bmstruckFormulaService;
    @Autowired
    ColumnDataUtil columnDataUtil;

    /**
     * 展示水运费和水分质检费账单信息
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示水运费和水分质检费账单信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "101", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getStatementList")
    public RESTfulResult getStatementList(@RequestBody(required = false) Map<String,Object> mapValue,
                                          Integer pageNum,
                                          Integer pageSize,
                                          Integer apiId) {
        List<Map<String, Object>> list = bmsshipStatementService.getStatementList(mapValue);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipStatementService.getStatementList(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

    /**
     * 展示途损费账单信息
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示途损费账单信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "101", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getLossStatementList")
    public RESTfulResult getLossStatementList(@RequestBody(required = false) Map<String,Object> mapValue,
                                          Integer pageNum,
                                          Integer pageSize,
                                          Integer apiId) {
        List<Map<String, Object>> list = bmsshipStatementService.getLossStatementList(mapValue);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipStatementService.getLossStatementList(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

    /**
     * 展示水分质检实绩
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示水分质检实绩")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "114", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getWaterResult")
    public RESTfulResult getWaterResult(@RequestBody(required = false) Map<String,Object> mapValue,
                                          Integer pageNum,
                                          Integer pageSize,
                                          Integer apiId) {
        List<Map<String, Object>> list = bmsshipStatementService.getWaterResult(mapValue);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipStatementService.getWaterResult(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

    /**
     * 展示批次信息
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示批次信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "115", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getBatch")
    public RESTfulResult getBatch(@RequestBody(required = false) Map<String,Object> mapValue,
                                        Integer pageNum,
                                        Integer pageSize,
                                        Integer apiId) {
        List<Map<String, Object>> list = bmsshipStatementService.getBatch(mapValue);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipStatementService.getBatch(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

    /**
     * 生成水运费账单
     * @param mapList
     * @return
     * @throws Exception
     */
    @ApiModelProperty(value = "生成水运费账单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mapList", value = "详单列表", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/addStatement")
    public RESTfulResult addStatement(@RequestBody List<Map<String,Object>> mapList) throws Exception {
        int code1 = bmsshipStatementService.insertSelective(mapList);
        return success(code1);
    }

    /**
     * 生成水分质检账单
     * @param mapList
     * @return
     * @throws Exception
     */
    @ApiModelProperty(value = "生成水分质检账单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mapList", value = "水分质检实绩列表", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/addWaterStatement")
    public RESTfulResult addWaterStatement(@RequestBody List<Map<String,Object>> mapList) throws Exception {
        int code = bmsshipStatementService.insertWaterFeesStatement(mapList);
        return success(code);
    }

    /**
     * 生成途损账单
     * @param map
     * @return
     * @throws Exception
     */
    @ApiModelProperty(value = "生成途损账单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "map", value = "批次", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/addLossStatement")
    public RESTfulResult addLossStatement(@RequestBody Map<String,Object> map) throws Exception {
        Integer batchId = (Integer) map.get("batchId");
        List<Map<String, Object>> calDataList = bmstruckFormulaService.getCalData(new BigDecimal(batchId));
        Map<String, Object> calDataMap = calDataList.get(0);
        BigDecimal shipWeight = (BigDecimal) calDataMap.get("shipWeight");
        BigDecimal moistureContent = (BigDecimal) calDataMap.get("moistureContent");
        BigDecimal count = (BigDecimal) calDataMap.get("count");
        BigDecimal arrivalWeight = bmstruckFormulaService.getArrivalWeight(new BigDecimal(batchId));
        Double result = (shipWeight.doubleValue() * (1 - 0.015) * (1 - moistureContent.doubleValue()/count.doubleValue())) - (arrivalWeight.doubleValue() * (1 - 0.091));
        if (result > 0) {
            bmsshipStatementService.insertLossStatement(new BigDecimal(batchId));
        }
        return success(result);
    }

    /**
     * 展示账单下的详单
     * @param batchId
     * @param mapVal
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiOperation(value="展示账单下的详单", notes="分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "batchId", value = "批次id", required = false, dataType = "BigDecimal"),
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "93", required = false, dataType = "BigDecimal"),
    })
    @PostMapping(value = "/getStatementDetailsOrderList")
    public RESTfulResult getStatementDetailsOrderList(BigDecimal batchId,@RequestBody(required = false) Map<String,Object> mapVal,
                                                      Integer pageNum,
                                                      Integer pageSize,
                                                      Integer apiId,
                                                      BigDecimal statementId){
        if (mapVal==null) {
            mapVal = new HashMap<>();
        }
        mapVal.put("batchId",batchId);
        mapVal.put("statementId",statementId);
        List<Map<String,Object>> list = bmsshipStatementService.getDetailsOrder(mapVal);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipStatementService.getDetailsOrder(mapVal);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

//    /**
//     * 展示账单下的所有费用
//     * @param batchId
//     * @return
//     */
//    @ApiOperation(value="展示账单下的所有费用")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "batchId", value = "账单id", required = false, dataType = "BigDecimal"),
//    })
//    @PostMapping(value = "/getAllCost")
//    public RESTfulResult getAllCost(BigDecimal batchId) throws Exception {
//        // 损失费
//        BigDecimal lossCost = bmstruckFormulaService.generateLossCost(batchId);
//        // 水分检测费
//        BigDecimal waterFees = new BigDecimal(bmstruckFormulaService.selectWaterFees(batchId));
//        // 总金额和支付单位
//        List<Map<String,Object>> listMap = bmsshipStatementService.getShipperAndAmount(batchId);
//        Map map = listMap.get(0);
//        // 总金额
//        BigDecimal totalAmount = (BigDecimal) map.get("statementTotalAmount");
//        Double shipCostDouble = (totalAmount.doubleValue() - lossCost.doubleValue() - waterFees.doubleValue());
//        DecimalFormat df = new DecimalFormat("0.00");
//        String shipCost = df.format(shipCostDouble);
//        String lossCostStr = df.format(lossCost.doubleValue());
//        map.put("lossCost",lossCostStr);
//        map.put("waterFees",waterFees);
//        map.put("shipCost",shipCost);
//        return success(listMap);
//    }

    /**
     * 展示详单下的船运实绩
     * @param batchId
     * @param mapVal
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiOperation(value="展示详单下的船运实绩", notes="分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "batchId", value = "账单id", required = false, dataType = "BigDecimal",paramType = "path"),
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "112", required = false, dataType = "BigDecimal"),
    })
    @PostMapping(value = "/getShipResult")
    public RESTfulResult getShipResult(BigDecimal batchId,@RequestBody(required = false) Map<String,Object> mapVal,
                                        Integer pageNum,
                                        Integer pageSize,
                                        Integer apiId){
        if (mapVal==null) {
            mapVal = new HashMap<>();
        }
        mapVal.put("batchId",batchId);
        List<Map<String,Object>> list = bmsshipStatementService.getShipResult(mapVal);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipStatementService.getShipResult(mapVal);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }



}
