package com.steerinfo.dil.mapper;

import com.steerinfo.dil.model.BmstruckFormula;
import com.steerinfo.framework.mapper.IBaseMapper;
import java.math.*;
import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Mapper;

@Mapper
public interface BmstruckFormulaMapper extends IBaseMapper<BmstruckFormula, BigDecimal> {

    /**
     * 查询价格日期
     * @return
     */
    List<Map<String,Object>> selectPriceDate();


    /**
     * 查询装船日期和装船比例
     * @return
     */
    List<Map<String,Object>> selectLoadShipDate(BigDecimal batchId);

    /**
     * 获取到厂湿吨
     * @param resultTotalId
     * @return
     */
    BigDecimal selectArrivalWeight(BigDecimal resultTotalId);

    /**
     * 获取批次id
     * @param weightTaskResultId
     * @return
     */
    BigDecimal selectBatchId(BigDecimal weightTaskResultId);

    /**
     * 根据批次id得到采购订单id
     * @param batchId
     * @return
     */
    BigDecimal selectPurchaseId(BigDecimal batchId);

    /**
     * 得到一个批次下的水分质检费总和
     * @param batchId
     * @return
     */
    Double selectWaterFees(BigDecimal batchId);

    /**
     * 通过出厂实绩查询总实绩id
     * @param resultId
     * @return
     */
    BigDecimal selectTotalId(BigDecimal resultId);

    /**
     * 得到计算因子
     * @param batchId
     * @return
     */
    List<Map<String,Object>> getCalData(BigDecimal batchId);

    /**
     * 查询到厂总湿重
     * @param batchId
     * @return
     */
    BigDecimal getArrivalWeight(BigDecimal batchId);

    /**
     * 得到单价
     * @param batchId
     * @return
     */
    BigDecimal getUnitPrice(BigDecimal batchId);

    /**
     * 根据计重实绩得到水运计费公式
     * @return
     */
    String selectFormula(BigDecimal formula);

    /**
     * 根据主键查询计费公式
     * @param formulaId
     * @return
     */
    Map<String,Object> selectAnyFormula(BigDecimal formulaId);

    /**
     * 查询港口装卸费单价
     * @param priceId
     * @return
     */
    BigDecimal getPriceValue(BigDecimal priceId);
}