package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.BmsshipDetailsOrderMapper;
import com.steerinfo.dil.model.BmsshipDetailsOrder;
import com.steerinfo.dil.service.IBmsshipDetailsOrderService;
import com.steerinfo.dil.util.NoUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/7 17:15
 * @Version:V2.0
 */
@Service
public class BmsshipDetailsOrderSerivceImpl implements IBmsshipDetailsOrderService {

    @Autowired
    BmsshipDetailsOrderMapper bmsshipDetailsOrderMapper;
    @Autowired
    BmstruckFormulaServiceImpl bmstruckFormulaService;
    @Autowired
    NoUtil noUtil;

    /**
     * 展示详单信息
     * @param mapValue
     * @return
     */
    @Override
    public List<Map<String, Object>> getDetailsOrderList(Map<String, Object> mapValue) {
        return bmsshipDetailsOrderMapper.getDetailsOrderList(mapValue);
    }

    /**
     * 通过出厂实绩查询新增详单
     * @param resultId
     * @return
     */
    @Override
    public int insertSelective(BigDecimal resultId) throws Exception {
        BmsshipDetailsOrder bmsshipDetailsOrder = new BmsshipDetailsOrder();
        // 根据出厂实绩获取到总实绩id
        BigDecimal resultTotalId = bmstruckFormulaService.selectTotalId(resultId);
        // 得到批次id
        BigDecimal batchId = bmstruckFormulaService.selectBatchId(resultTotalId);
        // 得到采购订单id
        BigDecimal purchaseOrderId = bmstruckFormulaService.selectPurchaseId(batchId);
        // 设置采购订单id
        bmsshipDetailsOrder.setPurchaseOrderId(purchaseOrderId);
        // 设置汽运总实绩id
        bmsshipDetailsOrder.setTruckResultTotalId(resultTotalId);
        // 得到最大id设置主键
        BigDecimal detailsId = selectMaxId();
        bmsshipDetailsOrder.setDetailsId(detailsId);
        // 设置详单号
        String detailsNo = noUtil.setResultNo("CYXD", detailsId);
        bmsshipDetailsOrder.setDetailsNo(detailsNo);
        // 设置详单时间
        bmsshipDetailsOrder.setDetailsTime(new Date());
        // 设置费用类型
        bmsshipDetailsOrder.setDetailsFeeType("水运费");
        // 计算出水运费金额
        BigDecimal amount = bmstruckFormulaService.generateDetailedOrder(resultId);
        DecimalFormat df = new DecimalFormat("0.00");
        String format = df.format(amount);
        // 设置金额
        bmsshipDetailsOrder.setDetailsAmount(new BigDecimal(format));
        // 设置是否已开账单
        bmsshipDetailsOrder.setWetherToStatement(new BigDecimal(0));
        bmsshipDetailsOrder.setInsertTime(new Date());
        bmsshipDetailsOrder.setInsertUsername("admin");
        bmsshipDetailsOrder.setUpdateTime(new Date());
        bmsshipDetailsOrder.setUpdateUsername("admin");
        bmsshipDetailsOrder.setInsertUpdateRemark("无");
        return bmsshipDetailsOrderMapper.insertSelective(bmsshipDetailsOrder);
    }

    /**
     * 得到最大id
     * @return
     */
    @Override
    public BigDecimal selectMaxId() {
        return bmsshipDetailsOrderMapper.selectMaxId() == null ? new BigDecimal(1) : bmsshipDetailsOrderMapper.selectMaxId();
    }

    /**
     * 主键查询
     * @param detailsId
     * @return
     */
    @Override
    public BmsshipDetailsOrder selectByPrimaryKey(BigDecimal detailsId) {
        return bmsshipDetailsOrderMapper.selectByPrimaryKey(detailsId);
    }

    /**
     * 修改详单状态
     * @param bmsshipDetailsOrder
     * @return
     */
    @Override
    public int updateByPrimaryKeySelective(BmsshipDetailsOrder bmsshipDetailsOrder) {
        return bmsshipDetailsOrderMapper.updateByPrimaryKeySelective(bmsshipDetailsOrder);
    }

    /**
     * 未生成账单的详单
     * @param mapValue
     * @return
     */
    @Override
    public List<Map<String, Object>> getUncomplateDetailsOrderList(Map<String, Object> mapValue) {
        return bmsshipDetailsOrderMapper.getUncomplateDetailsOrderList(mapValue);
    }
}
